@@ -1,7 +1,7 @@
 #include "common.h"
 #include "cfg.h"
 
-int loadcfg(const char *cfgfile)
+int loadcfg(const char *cfgfile, const ConfigType type)
 {
 	FILE *fd;
 	int i;
@@ -57,6 +57,7 @@ int loadcfg(const char *cfgfile)
 		 {"PollInterval", 0, &cfg.pollinterval, 0, 0},
 		 {"SaveInterval", 0, &cfg.saveinterval, 0, 0},
 		 {"OfflineSaveInterval", 0, &cfg.offsaveinterval, 0, 0},
+		 {"AlwaysAddNewInterfaces", 0, &cfg.alwaysadd, 0, 0},
 		 {"BandwidthDetectionInterval", 0, &cfg.bwdetectioninterval, 0, 0},
 		 {"SaveOnStatusChange", 0, &cfg.savestatus, 0, 0},
 		 {"UseLogging", 0, &cfg.uselogging, 0, 0},
@@ -67,6 +68,7 @@ int loadcfg(const char *cfgfile)
 		 {"64bitInterfaceCounters", 0, &cfg.is64bit, 0, 0},
 		 {"DatabaseWriteAheadLogging", 0, &cfg.waldb, 0, 0},
 		 {"DatabaseSynchronous", 0, &cfg.dbsynchronous, 0, 0},
+		 {"UseUTC", 0, &cfg.useutc, 0, 0},
 		 {"HeaderFormat", cfg.hformat, 0, 64, 0},
 		 {"HourlyRate", 0, &cfg.hourlyrate, 0, 0},
 		 {"SummaryRate", 0, &cfg.summaryrate, 0, 0},
@@ -149,7 +151,7 @@ int loadcfg(const char *cfgfile)
 	fclose(fd);
 
 	/* validate config */
-	validatecfg();
+	validatecfg(type);
 
 	return 1;
 }
@@ -163,25 +165,25 @@ void validateint(const char *cfgname, int32_t *cfgptr, const int32_t defaultvalu
 {
 	if (maxvalue > minvalue) {
 		if (*cfgptr < minvalue || *cfgptr > maxvalue) {
-			*cfgptr = defaultvalue;
-			snprintf(errorstring, 1024, "Invalid value for %s, resetting to \"%d\".", cfgname, *cfgptr);
+			snprintf(errorstring, 1024, "Invalid value \"%d\" for %s. Value needs to be between %d and %d. Using default value %d.", *cfgptr, cfgname, minvalue, maxvalue, defaultvalue);
 			printe(PT_Config);
+			*cfgptr = defaultvalue;
 		}
 	} else {
 		if (*cfgptr < minvalue) {
-			*cfgptr = defaultvalue;
-			snprintf(errorstring, 1024, "Invalid value for %s, resetting to \"%d\".", cfgname, *cfgptr);
+			snprintf(errorstring, 1024, "Invalid value \"%d\" for %s. Value needs to be at least %d. Using default value %d.", *cfgptr, cfgname, minvalue, defaultvalue);
 			printe(PT_Config);
+			*cfgptr = defaultvalue;
 		}
 	}
 }
 
-void validatecfg(void)
+void validatecfg(const ConfigType type)
 {
 	uint32_t rolloversecs;
 	const char *invalidvalue = "Invalid value for";
-	const char *resettingto = "resetting to";
-	const char *noslashstart = "doesn't start with \"/\", resetting to default.";
+	const char *resettingto = "using default value";
+	const char *noslashstart = "doesn't start with \"/\", using default value instead.";
 
 	validateint("UnitMode", &cfg.unitmode, UNITMODE, 0, 2);
 	validatebool("RateUnitMode", &cfg.rateunitmode, RATEUNITMODE);
@@ -205,17 +207,20 @@ void validatecfg(void)
 	validateint("64bitInterfaceCounters", &cfg.is64bit, IS64BIT, -2, 1);
 	validatebool("DatabaseWriteAheadLogging", &cfg.waldb, WALDB);
 	validateint("DatabaseSynchronous", &cfg.dbsynchronous, DBSYNCHRONOUS, -1, 3);
-	validatebool("TransparentBg", &cfg.transbg, TRANSBG);
-	validatebool("LargeFonts", &cfg.largefonts, LARGEFONTS);
-	validateint("LineSpacingAdjustment", &cfg.linespaceadjust, LINESPACEADJUST, -5, 10);
-	validateint("ImageScale", &cfg.imagescale, IMAGESCALE, 50, 500);
-	validateint("5MinuteGraphResultCount", &cfg.fivegresultcount, FIVEGRESULTCOUNT, FIVEGMINRESULTCOUNT, 2000);
-	validateint("5MinuteGraphHeight", &cfg.fivegheight, FIVEGHEIGHT, FIVEGMINHEIGHT, 2000);
-	validatebool("SummaryGraph", &cfg.summarygraph, SUMMARYGRAPH);
-	validateint("EstimateStyle", &cfg.estimatestyle, ESTIMATESTYLE, 0, 2);
-	validatebool("BarColumnShowsRate", &cfg.barshowsrate, BARSHOWSRATE);
-	validatebool("HourlyRate", &cfg.hourlyrate, HOURLYRATE);
-	validatebool("SummaryRate", &cfg.summaryrate, SUMMARYRATE);
+	validatebool("UseUTC", &cfg.useutc, USEUTC);
+	if (type == CT_Image || type == CT_All) {
+		validatebool("TransparentBg", &cfg.transbg, TRANSBG);
+		validatebool("LargeFonts", &cfg.largefonts, LARGEFONTS);
+		validateint("LineSpacingAdjustment", &cfg.linespaceadjust, LINESPACEADJUST, -5, 10);
+		validateint("ImageScale", &cfg.imagescale, IMAGESCALE, 50, 500);
+		validateint("5MinuteGraphResultCount", &cfg.fivegresultcount, FIVEGRESULTCOUNT, FIVEGMINRESULTCOUNT, 2000);
+		validateint("5MinuteGraphHeight", &cfg.fivegheight, FIVEGHEIGHT, FIVEGMINHEIGHT, 2000);
+		validatebool("SummaryGraph", &cfg.summarygraph, SUMMARYGRAPH);
+		validateint("EstimateStyle", &cfg.estimatestyle, ESTIMATESTYLE, 0, 2);
+		validatebool("BarColumnShowsRate", &cfg.barshowsrate, BARSHOWSRATE);
+		validatebool("HourlyRate", &cfg.hourlyrate, HOURLYRATE);
+		validatebool("SummaryRate", &cfg.summaryrate, SUMMARYRATE);
+	}
 	validatebool("TrafficlessEntries", &cfg.trafficlessentries, TRAFFICLESSENTRIES);
 	validateint("List5Mins", &cfg.listfivemins, LISTFIVEMINS, 0, 0);
 	validateint("ListHours", &cfg.listhours, LISTHOURS, 0, 0);
@@ -233,6 +238,12 @@ void validatecfg(void)
 	validateint("BandwidthDetectionInterval", &cfg.bwdetectioninterval, BWDETECTINTERVAL, 0, 30);
 	validatebool("Experimental", &cfg.experimental, 0);
 
+	if (cfg.useutc) {
+		strncpy_nt(cfg.dbtzmodifier, "", 14);
+	} else {
+		strncpy_nt(cfg.dbtzmodifier, DATABASELOCALTIMEMODIFIER, 14);
+	}
+
 	if (cfg.dbdir[0] != '/') {
 		strncpy_nt(cfg.dbdir, DATABASEDIR, 512);
 		snprintf(errorstring, 1024, "DatabaseDir %s", noslashstart);
@@ -251,63 +262,74 @@ void validatecfg(void)
 		printe(PT_Config);
 	}
 
-	if (cfg.updateinterval < cfg.pollinterval || cfg.updateinterval > 300) {
-		if (cfg.pollinterval > UPDATEINTERVAL) {
-			cfg.updateinterval = cfg.pollinterval;
-		} else {
-			cfg.updateinterval = UPDATEINTERVAL;
-		}
-		snprintf(errorstring, 1024, "%s UpdateInterval, %s \"%d\".", invalidvalue, resettingto, cfg.updateinterval);
-		printe(PT_Config);
-	}
+	if (type == CT_Daemon || type == CT_All) {
+		validatebool("RescanDatabaseOnSave", &cfg.rescanonsave, RESCANONSAVE);
+		validatebool("AlwaysAddNewInterfaces", &cfg.alwaysadd, ALWAYSADD);
 
-	if ((cfg.saveinterval * 60) < cfg.updateinterval || cfg.saveinterval > 60) {
-		if (cfg.updateinterval > (SAVEINTERVAL * 60)) {
-			cfg.saveinterval = cfg.updateinterval;
-		} else {
-			cfg.saveinterval = SAVEINTERVAL;
-		}
-		snprintf(errorstring, 1024, "%s SaveInterval, %s \"%d\".", invalidvalue, resettingto, cfg.saveinterval);
-		printe(PT_Config);
-	}
-
-	if (cfg.offsaveinterval < cfg.saveinterval || cfg.offsaveinterval > 60) {
-		if (cfg.saveinterval > OFFSAVEINTERVAL) {
-			cfg.offsaveinterval = cfg.saveinterval;
-		} else {
-			cfg.offsaveinterval = OFFSAVEINTERVAL;
-		}
-		snprintf(errorstring, 1024, "%s OfflineSaveInterval, %s \"%d\".", invalidvalue, resettingto, cfg.offsaveinterval);
-		printe(PT_Config);
-	}
-
-	/* enforce update interval to be short enough that 32-bit interface counter rollover can be detected */
-	/* 1.02 is the same 2% safety buffer as used in processifinfo() in daemon.c */
-	/* noexit check results in warning being shown only when the daemon is started */
-	if (noexit && cfg.maxbw > 0) {
-		rolloversecs = (uint32_t)((float)MAX32 / ((float)cfg.maxbw * 1024 * 1024 * (float)1.02 / 8));
-		if (rolloversecs <= (uint32_t)cfg.updateinterval) {
-			cfg.updateinterval = UPDATEINTERVAL;
-			if (rolloversecs <= (uint32_t)cfg.updateinterval) {
-				cfg.updateinterval /= 2;
+		if (cfg.updateinterval < cfg.pollinterval || cfg.updateinterval > 300) {
+			if (cfg.pollinterval > UPDATEINTERVAL) {
+				cfg.updateinterval = cfg.pollinterval;
+			} else {
+				cfg.updateinterval = UPDATEINTERVAL;
 			}
-			snprintf(errorstring, 1024, "UpdateInterval has been reset to %d seconds in order to ensure correct counter rollover detection at %d Mbit.", cfg.updateinterval, cfg.maxbw);
+			snprintf(errorstring, 1024, "%s UpdateInterval, %s %d.", invalidvalue, resettingto, cfg.updateinterval);
 			printe(PT_Config);
 		}
+
+		if ((cfg.saveinterval * 60) < cfg.updateinterval || cfg.saveinterval > 60) {
+			if (cfg.updateinterval > (SAVEINTERVAL * 60)) {
+				cfg.saveinterval = cfg.updateinterval;
+			} else {
+				cfg.saveinterval = SAVEINTERVAL;
+			}
+			snprintf(errorstring, 1024, "%s SaveInterval, %s %d.", invalidvalue, resettingto, cfg.saveinterval);
+			printe(PT_Config);
+		}
+
+		if (cfg.offsaveinterval < cfg.saveinterval || cfg.offsaveinterval > 60) {
+			if (cfg.saveinterval > OFFSAVEINTERVAL) {
+				cfg.offsaveinterval = cfg.saveinterval;
+			} else {
+				cfg.offsaveinterval = OFFSAVEINTERVAL;
+			}
+			snprintf(errorstring, 1024, "%s OfflineSaveInterval, %s %d.", invalidvalue, resettingto, cfg.offsaveinterval);
+			printe(PT_Config);
+		}
+
+		/* enforce update interval to be short enough that 32-bit interface counter rollover can be detected */
+		/* 1.02 is the same 2% safety buffer as used in processifinfo() in daemon.c */
+		if (cfg.maxbw > 0) {
+			rolloversecs = (uint32_t)((float)MAX32 / ((float)cfg.maxbw * 1024 * 1024 * (float)1.02 / 8));
+			if (rolloversecs <= (uint32_t)cfg.updateinterval) {
+				cfg.updateinterval = UPDATEINTERVAL;
+				if (rolloversecs <= (uint32_t)cfg.updateinterval) {
+					cfg.updateinterval /= 2;
+				}
+				snprintf(errorstring, 1024, "UpdateInterval has been reset to %d seconds in order to ensure correct counter rollover detection at %d Mbit.", cfg.updateinterval, cfg.maxbw);
+				printe(PT_Config);
+			}
+		}
 	}
 
 	/* affects only image output */
-	if (cfg.barshowsrate && cfg.estimatebarvisible) {
-		cfg.estimatestyle = 0;
-		if (debug) {
-			printf("BarColumnShowsRate and EstimateBarVisible both enabled -> EstimateStyle set to 0\n");
+	if (type == CT_Image || type == CT_All) {
+		if (cfg.barshowsrate && cfg.estimatebarvisible) {
+			cfg.estimatestyle = 0;
+			if (debug) {
+				printf("BarColumnShowsRate and EstimateBarVisible both enabled -> EstimateStyle set to 0\n");
+			}
 		}
-	}
 
-	if (cfg.fiveminutehours > 0 && cfg.fivegresultcount > cfg.fiveminutehours * 12) {
-		cfg.fivegresultcount = cfg.fiveminutehours * 12;
-		snprintf(errorstring, 1024, "5MinuteGraphResultCount has been reset to %d due to request being larger than data retention configured with 5MinuteHours.", cfg.fivegresultcount);
-		printe(PT_Config);
+		if (cfg.fiveminutehours > 0 && cfg.fivegresultcount > cfg.fiveminutehours * 12) {
+			if (cfg.fiveminutehours * 12 < FIVEGMINRESULTCOUNT) {
+				snprintf(errorstring, 1024, "Value \"%d\" for 5MinuteHours is too small for 5MinuteGraphResultCount with value \"%d\" (smallest supported value: %d). Value for 5MinuteHours needs to be at least %d if 5MinuteGraphResultCount is set to %d.", cfg.fiveminutehours, cfg.fivegresultcount, FIVEGMINRESULTCOUNT, FIVEGMINRESULTCOUNT / 12, FIVEGMINRESULTCOUNT);
+				cfg.fivegresultcount = FIVEGMINRESULTCOUNT;
+			} else {
+				snprintf(errorstring, 1024, "5MinuteGraphResultCount has been adjusted to %d because requested \"%d\" requires more data than can be available with value %d for 5MinuteHours.", cfg.fiveminutehours * 12, cfg.fivegresultcount, cfg.fiveminutehours);
+				cfg.fivegresultcount = cfg.fiveminutehours * 12;
+			}
+			printe(PT_Config);
+		}
 	}
 }
 
@@ -353,6 +375,7 @@ void defaultcfg(void)
 	cfg.topdayentries = TOPDAYENTRIES;
 
 	strncpy_nt(cfg.dbdir, DATABASEDIR, 512);
+	strncpy_nt(cfg.dbtzmodifier, DATABASELOCALTIMEMODIFIER, 14);
 	strncpy_nt(cfg.iface, DEFIFACE, 32);
 	strncpy_nt(cfg.locale, LOCALE, 32);
 	strncpy_nt(cfg.dformat, DFORMAT, 64);
@@ -371,6 +394,8 @@ void defaultcfg(void)
 	cfg.pollinterval = POLLINTERVAL;
 	cfg.saveinterval = SAVEINTERVAL;
 	cfg.offsaveinterval = OFFSAVEINTERVAL;
+	cfg.rescanonsave = RESCANONSAVE;
+	cfg.alwaysadd = ALWAYSADD;
 	cfg.savestatus = SAVESTATUS;
 	cfg.uselogging = USELOGGING;
 	cfg.createdirs = CREATEDIRS;
@@ -380,6 +405,7 @@ void defaultcfg(void)
 	cfg.is64bit = IS64BIT;
 	cfg.waldb = WALDB;
 	cfg.dbsynchronous = DBSYNCHRONOUS;
+	cfg.useutc = USEUTC;
 
 	cfg.transbg = TRANSBG;
 	cfg.largefonts = LARGEFONTS;
