@@ -9,6 +9,10 @@ void printcfgfile(void)
 	printf("# vnStat %s config file\n", getversion());
 	printf("##\n\n");
 
+	if (cfg.experimental) {
+		printf("Experimental %d\n\n", cfg.experimental);
+	}
+
 	printf("# default interface (leave empty for automatic selection)\n");
 	printf("Interface \"%s\"\n\n", cfg.iface);
 
@@ -47,7 +51,8 @@ void printcfgfile(void)
 	printf("# 0 = minimal & narrow, 1 = bar column visible\n");
 	printf("# 2 = same as 1 except rate in summary\n");
 	printf("# 3 = rate column visible\n");
-	printf("OutputStyle %d\n\n", cfg.ostyle);
+	printf("OutputStyle %d\n", cfg.ostyle);
+	printf("EstimateBarVisible %d\n\n", cfg.estimatebarvisible);
 
 	printf("# number of decimals to use in outputs\n");
 	printf("DefaultDecimals %d\n", cfg.defaultdecimals);
@@ -121,6 +126,13 @@ void printcfgfile(void)
 	printf("# how often (in minutes) data is saved when all interface are offline\n");
 	printf("OfflineSaveInterval %d\n\n", cfg.offsaveinterval);
 
+	printf("# rescan database after save for new interfaces to be monitored (1 = enabled, 0 = disabled)\n");
+	printf("RescanDatabaseOnSave %d\n\n", cfg.rescanonsave);
+
+	printf("# automatically start monitoring all interfaces not found in the database\n");
+	printf("# (1 = enabled, 0 = disabled)\n");
+	printf("AlwaysAddNewInterfaces %d\n\n", cfg.alwaysadd);
+
 	printf("# on which day should months change\n");
 	printf("MonthRotate %d\n", cfg.monthrotate);
 	printf("MonthRotateAffectsYears %d\n\n", cfg.monthrotateyears);
@@ -170,6 +182,9 @@ void printcfgfile(void)
 	printf("# (-1 = auto, 0 = off, 1, = normal, 2 = full, 3 = extra)\n");
 	printf("DatabaseSynchronous %d\n\n", cfg.dbsynchronous);
 
+	printf("# database uses UTC instead of local timezone (1 = enabled, 0 = disabled)\n");
+	printf("UseUTC %d\n\n", cfg.useutc);
+
 	printf("\n");
 
 	/* vnstati section */
@@ -187,6 +202,26 @@ void printcfgfile(void)
 	printf("# transparent background (1 = enabled, 0 = disabled)\n");
 	printf("TransparentBg %d\n\n", cfg.transbg);
 
+	printf("# image size control\n");
+	printf("LargeFonts %d\n", cfg.largefonts);
+	printf("LineSpacingAdjustment %d\n", cfg.linespaceadjust);
+	printf("ImageScale %d\n\n", cfg.imagescale);
+
+	printf("# 5 minutes graph size control\n");
+	printf("5MinuteGraphResultCount %d\n", cfg.fivegresultcount);
+	printf("5MinuteGraphHeight %d\n\n", cfg.fivegheight);
+
+	printf("# horizontal/vertical summary graph (0 = hours, 1 = 5 minutes)\n");
+	printf("SummaryGraph %d\n\n", cfg.summarygraph);
+
+	printf("# traffic estimate bar style\n");
+	printf("# (0 = not shown, 1 = continuation of existing bar, 2 = separate bar)\n");
+	printf("EstimateStyle %d\n\n", cfg.estimatestyle);
+
+	printf("# bar column in list outputs shows rate if OutputStyle is 3\n");
+	printf("# (1 = enabled, 0 = disabled)\n");
+	printf("BarColumnShowsRate %d\n\n", cfg.barshowsrate);
+
 	printf("# image colors\n");
 	printf("CBackground     \"%s\"\n", cfg.cbg);
 	printf("CEdge           \"%s\"\n", cfg.cedge);
