@@ -71,6 +71,9 @@ and most can be changed later from the config file.
 /* database file name */
 #define DATABASEFILE "vnstat.db"
 
+/* modifier used for sqlite date and time functions */
+#define DATABASELOCALTIMEMODIFIER ", 'localtime'"
+
 /* on which day should months change */
 #define MONTHROTATE 1
 #define MONTHROTATEYEARS 0
@@ -105,6 +108,7 @@ and most can be changed later from the config file.
 /* 2 = same as 1 + rate in summary */
 /* 3 = rate everywhere */
 #define OSTYLE 3
+#define ESTIMATEBARVISIBLE 1
 
 /* rate in vnstati summary output */
 #define SUMMARYRATE 1
@@ -225,6 +229,8 @@ and most can be changed later from the config file.
 #define POLLINTERVAL 5
 #define SAVEINTERVAL 5
 #define OFFSAVEINTERVAL 30
+#define RESCANONSAVE 1
+#define ALWAYSADD 0
 #define SAVESTATUS 1
 #define USELOGGING 2
 #define CREATEDIRS 1
@@ -236,6 +242,7 @@ and most can be changed later from the config file.
 #define WALDBCHECKPOINTINTERVALMINS 240
 #define SLOWDBWARNLIMIT 4.0 // needs to be less than DBREADTIMEOUTSECS
 #define DBSYNCHRONOUS -1
+#define USEUTC 0
 
 /* database read timeout */
 #define DBREADTIMEOUTSECS 5
@@ -243,6 +250,31 @@ and most can be changed later from the config file.
 /* no transparency by default */
 #define TRANSBG 0
 
+/* small fonts by default */
+#define LARGEFONTS 0
+
+/* no extra space between lines by default */
+#define LINESPACEADJUST 0
+
+/* no image scaling by default */
+#define IMAGESCALE 100
+
+/* image output estimate bar style */
+/* 0 = not shown, 1 = continuation of existing bar, 2 = separate bar */
+#define ESTIMATESTYLE 1
+
+/* bar column in list outputs shows rate when rate column is visible */
+#define BARSHOWSRATE 0
+
+/* 5 minute graph size */
+#define FIVEGRESULTCOUNT 576
+#define FIVEGHEIGHT 300
+#define FIVEGMINRESULTCOUNT 288
+#define FIVEGMINHEIGHT 150
+
+/* graph used in extended summary, 0 = hours, 1 = 5 minutes*/
+#define SUMMARYGRAPH 0
+
 /* default colors */
 #define CBACKGROUND "FFFFFF"
 #define CEDGE "AEAEAE"
@@ -267,19 +299,22 @@ typedef struct {
 	char dformat[64], mformat[64], tformat[64], hformat[64];
 	char iface[32];
 	char locale[32];
-	char dbdir[512];
+	char dbdir[512], dbtzmodifier[14];
 	char rxchar[2], txchar[2], rxhourchar[2], txhourchar[2];
 	char cbg[8], cedge[8], cheader[8], cheadertitle[8], cheaderdate[8], ctext[8];
 	char cline[8], clinel[8], cvnstat[8], crx[8], crxd[8], ctx[8], ctxd[8];
 	int32_t unitmode, rateunitmode, rateunit, bvar, qmode, sampletime, hourlyrate, summaryrate;
 	int32_t monthrotate, monthrotateyears, maxbw, spacecheck, trafficlessentries, transbg, ostyle;
-	int32_t defaultdecimals, hourlydecimals, hourlystyle, is64bit, waldb, dbsynchronous;
+	int32_t defaultdecimals, hourlydecimals, hourlystyle, is64bit, waldb, dbsynchronous, useutc, imagescale;
+	int32_t largefonts, linespaceadjust, estimatebarvisible, estimatestyle, barshowsrate, fivegresultcount;
+	int32_t fivegheight, summarygraph, alwaysadd;
 	char cfgfile[512], logfile[512], pidfile[512];
 	char daemonuser[33], daemongroup[33];
-	int32_t timesyncwait, updateinterval, pollinterval, saveinterval, offsaveinterval, savestatus;
+	int32_t timesyncwait, updateinterval, pollinterval, saveinterval, offsaveinterval, rescanonsave, savestatus;
 	int32_t uselogging, createdirs, updatefileowner, bwdetection, bwdetectioninterval, utflocale;
 	int32_t fiveminutehours, hourlydays, dailydays, monthlymonths, yearlyyears, topdayentries;
 	int32_t listfivemins, listhours, listdays, listmonths, listyears, listtop, listjsonxml;
+	int32_t experimental;
 } CFG;
 
 /* internal interface information structure */
