@@ -6,7 +6,7 @@ void showjson(const char *interface, const int ifcount, const char mode, const c
 {
 	interfaceinfo info;
 
-	timeused(__func__, 1);
+	timeused_debug(__func__, 1);
 
 	if (!db_getinterfacecountbyname(interface)) {
 		return;
@@ -70,7 +70,7 @@ void showjson(const char *interface, const int ifcount, const char mode, const c
 
 	printf("}}");
 
-	timeused(__func__, 0);
+	timeused_debug(__func__, 0);
 }
 
 void jsondump(const interfaceinfo *interface, const char *tablename, const int datetype, const char *databegin, const char *dataend)
