@@ -164,6 +164,11 @@ void showsummary(const interfaceinfo *interface, const int shortmode)
 		datalist_i = datalist_i->next;
 	}
 
+	if (!datalist) {
+		indent(5 + 27);
+		printf("no data available\n");
+	}
+
 	if (!shortmode) {
 		indent(5);
 		if (cfg.ostyle >= 2) {
@@ -253,6 +258,10 @@ void showsummary(const interfaceinfo *interface, const int shortmode)
 	}
 
 	if (!shortmode) {
+		if (!datalist) {
+			indent(5 + 27);
+			printf("no data available\n");
+		}
 		indent(5);
 		if (cfg.ostyle >= 2) {
 			printf("------------------------+-------------+-------------+---------------\n");
@@ -339,11 +348,20 @@ void showlist(const interfaceinfo *interface, const char *listname, const char *
 
 	daybuff[0] = '\0';
 
-	if (!db_getdata_range(&datalist, &datainfo, interface->name, listname, (uint32_t)limit, databegin, dataend) || !datalist) {
+	if (!db_getdata_range(&datalist, &datainfo, interface->name, listname, (uint32_t)limit, databegin, dataend)) {
 		printf("Error: Failed to fetch %s data.\n", titlename);
 		return;
 	}
 
+	if (!datalist) {
+		printf("No %s data available", titlename);
+		if (strlen(databegin) || strlen(dataend)) {
+			printf(" for given date range");
+		}
+		printf(".\n");
+		return;
+	}
+
 	datalist_i = datalist;
 
 	if (strlen(dataend) == 0 && datainfo.count > 0 && (listtype == LT_Day || listtype == LT_Month || listtype == LT_Year)) {
