@@ -37,7 +37,7 @@ int spacecheck(const char *path)
 		}
 	}
 
-	free = (uint64_t)(buf.f_bavail / (float)1024) * buf.f_bsize;
+	free = (uint64_t)((double)buf.f_bavail / (double)1024) * buf.f_bsize;
 
 	if (debug) {
 		printf("bsize %d\n", (int)buf.f_bsize);
@@ -170,12 +170,12 @@ char *getvalue(const uint64_t bytes, const int len, const RequestType type)
 			limit = (uint64_t)(pow(p, i - 1)) * 1000;
 			if (bytes >= limit) {
 				if (i > 1) {
-					snprintf(buffer, 64, "%" DECCONV "*.*f %s", getunitspacing(len, 5), declen, bytes / (double)(getunitdivisor(cfg.unitmode, i + 1)), getunitprefix(i + 1));
+					snprintf(buffer, 64, "%" DECCONV "*.*f %s", getunitspacing(len, 5), declen, (double)bytes / (double)(getunitdivisor(cfg.unitmode, i + 1)), getunitprefix(i + 1));
 				} else {
 					if (type == RT_Estimate) {
 						declen = 0;
 					}
-					snprintf(buffer, 64, "%" DECCONV "*.*f %s", getunitspacing(len, 2), declen, bytes / (double)(getunitdivisor(cfg.unitmode, i + 1)), getunitprefix(i + 1));
+					snprintf(buffer, 64, "%" DECCONV "*.*f %s", getunitspacing(len, 2), declen, (double)bytes / (double)(getunitdivisor(cfg.unitmode, i + 1)), getunitprefix(i + 1));
 				}
 				return buffer;
 			}
@@ -295,13 +295,13 @@ char *getratestring(const uint64_t rate, const int len, const int declen)
 		limit = (uint64_t)(pow(p, i - 1)) * 1000;
 		if (rate >= limit) {
 			l = getratespacing(len, unit, i + 1);
-			snprintf(buffer, 64, "%" DECCONV "*.*f %s", l, declen, rate / (double)(getunitdivisor(unit, i + 1)), getrateunitprefix(unit, i + 1));
+			snprintf(buffer, 64, "%" DECCONV "*.*f %s", l, declen, (double)rate / (double)(getunitdivisor(unit, i + 1)), getrateunitprefix(unit, i + 1));
 			return buffer;
 		}
 	}
 
 	l = getratespacing(len, unit, 1);
-	snprintf(buffer, 64, "%" DECCONV "*.0f %s", l, rate / (double)(getunitdivisor(unit, 1)), getrateunitprefix(unit, 1));
+	snprintf(buffer, 64, "%" DECCONV "*.0f %s", l, (double)rate / (double)(getunitdivisor(unit, 1)), getrateunitprefix(unit, 1));
 	return buffer;
 }
 
@@ -445,3 +445,118 @@ int issametimeslot(const ListType listtype, const time_t entry, const time_t upd
 
 	return 0;
 }
+
+uint64_t getperiodseconds(const ListType listtype, const time_t entry, const time_t updated, const short isongoing)
+{
+	struct tm e, u;
+	uint64_t seconds = 0;
+
+	if (localtime_r(&entry, &e) == NULL || localtime_r(&updated, &u) == NULL) {
+		return 0;
+	}
+
+	if (isongoing) {
+		if (listtype == LT_Day) {
+			seconds = (uint64_t)u.tm_sec + (uint64_t)u.tm_min * 60 + (uint64_t)u.tm_hour * 3600;
+		} else if (listtype == LT_Month) {
+			seconds = (uint64_t)mosecs(entry, updated);
+		} else if (listtype == LT_Year) {
+			seconds = (uint64_t)u.tm_yday * 86400 + (uint64_t)u.tm_sec + (uint64_t)u.tm_min * 60 + (uint64_t)u.tm_hour * 3600;
+		} else if (listtype == LT_Top) {
+			seconds = 86400;
+		} else if (listtype == LT_Hour) {
+			seconds = (uint64_t)u.tm_sec + (uint64_t)u.tm_min * 60;
+		} else if (listtype == LT_5min) {
+			seconds = (uint64_t)u.tm_sec + (uint64_t)u.tm_min % 5 * 60;
+		}
+	} else {
+		if (listtype == LT_Day || listtype == LT_Top) {
+			seconds = 86400;
+		} else if (listtype == LT_Month) {
+			seconds = (uint64_t)dmonth(e.tm_mon) * 86400;
+		} else if (listtype == LT_Year) {
+			seconds = (uint64_t)(365 + isleapyear(e.tm_year + 1900)) * 86400;
+		} else if (listtype == LT_Hour) {
+			seconds = 3600;
+		} else if (listtype == LT_5min) {
+			seconds = 300;
+		}
+	}
+
+	return seconds;
+}
+
+void getestimates(uint64_t *rx, uint64_t *tx, const ListType listtype, const time_t updated, dbdatalist **dbdata)
+{
+	struct tm u;
+	uint64_t div = 0, mult = 0;
+	dbdatalist *datalist_i = *dbdata;
+
+	*rx = *tx = 0;
+
+	if (datalist_i == NULL) {
+		return;
+	}
+
+	if (localtime_r(&updated, &u) == NULL) {
+		return;
+	}
+
+	/* last entry on the list is the most recent entry */
+	while (datalist_i->next != NULL) {
+		datalist_i = datalist_i->next;
+	}
+
+	if (datalist_i->rx == 0 || datalist_i->tx == 0) {
+		return;
+	}
+
+	/* LT_5min and LT_Hour don't have the estimate line visible in outputs */
+	/* but are used by BarColumnShowsRate which requires "past" values for */
+	/* full hours / 5 minutes for the bar to show correctly */
+	if (listtype == LT_5min) {
+		div = ((uint64_t)u.tm_min % 5 * 60) + (uint64_t)u.tm_sec;
+		if (div == 0) {
+			div = 1;
+			mult = 1;
+		} else {
+			mult = 300;
+		}
+	} else if (listtype == LT_Hour) {
+		div = (uint64_t)u.tm_min * 60 + (uint64_t)u.tm_sec;
+		if (div == 0) {
+			div = 1;
+			mult = 1;
+		} else {
+			mult = 3600;
+		}
+	} else if (listtype == LT_Day) {
+		div = (uint64_t)u.tm_hour * 3600 + (uint64_t)u.tm_min * 60 + (uint64_t)u.tm_sec;
+		mult = 86400;
+	} else if (listtype == LT_Month) {
+		div = (uint64_t)mosecs(datalist_i->timestamp, updated);
+		mult = (uint64_t)dmonth(u.tm_mon) * 86400;
+	} else if (listtype == LT_Year) {
+		div = (uint64_t)u.tm_yday * 1440 + (uint64_t)u.tm_hour * 60 + (uint64_t)u.tm_min;
+		mult = (uint64_t)(365 + isleapyear(u.tm_year + 1900)) * 1440;
+	}
+	if (div > 0) {
+		*rx = (uint64_t)((double)datalist_i->rx / (double)div) * mult;
+		*tx = (uint64_t)((double)datalist_i->tx / (double)div) * mult;
+	}
+}
+
+int ishelprequest(const char *arg)
+{
+	if (strlen(arg) == 0) {
+		return 0;
+	}
+
+	if (strlen(arg) == 1 && arg[0] == '?') {
+		return 1;
+	} else if ((strcmp(arg, "-?") == 0) || (strcmp(arg, "--help") == 0)) {
+		return 1;
+	}
+
+	return 0;
+}
