@@ -141,7 +141,7 @@ void parseargs(PARAMS *p, int argc, char **argv)
 		} else if ((strcmp(argv[currentarg], "-?") == 0) || (strcmp(argv[currentarg], "--help") == 0)) {
 			showhelp(p);
 			exit(EXIT_SUCCESS);
-		} else if ((strcmp(argv[currentarg], "-i") == 0) || (strcmp(argv[currentarg], "--iface") == 0)) {
+		} else if ((strcmp(argv[currentarg], "-i") == 0) || (strcmp(argv[currentarg], "--iface") == 0) || (strcmp(argv[currentarg], "--interface") == 0)) {
 			if (currentarg + 1 < argc) {
 				if (strlen(argv[currentarg + 1]) > 31) {
 					printf("Error: Interface name is limited to 31 characters.\n");
@@ -181,14 +181,8 @@ void parseargs(PARAMS *p, int argc, char **argv)
 			if (currentarg + 1 < argc && isdigit(argv[currentarg + 1][0])) {
 				cfg.ostyle = atoi(argv[currentarg + 1]);
 				if (cfg.ostyle > 4 || cfg.ostyle < 0) {
-					printf("Error: Invalid style parameter \"%d\" for --style.\n", cfg.ostyle);
-					printf(" Valid parameters:\n");
-					printf("    0 - a more narrow output\n");
-					printf("    1 - enable bar column if available\n");
-					printf("    2 - average traffic rate in summary output\n");
-					printf("    3 - average traffic rate in all outputs if available\n");
-					printf("    4 - disable terminal control characters in -l / --live\n");
-					printf("        and show raw values in --oneline\n");
+					printf("Error: Invalid style parameter \"%d\"\n", cfg.ostyle);
+					showstylehelp();
 					exit(EXIT_FAILURE);
 				}
 				if (debug)
@@ -196,13 +190,7 @@ void parseargs(PARAMS *p, int argc, char **argv)
 				currentarg++;
 			} else {
 				printf("Error: Style parameter for --style missing.\n");
-				printf(" Valid parameters:\n");
-				printf("    0 - a more narrow output\n");
-				printf("    1 - enable bar column if available\n");
-				printf("    2 - average traffic rate in summary output\n");
-				printf("    3 - average traffic rate in all outputs if available\n");
-				printf("    4 - disable terminal control characters in -l / --live\n");
-				printf("        and show raw values in --oneline\n");
+				showstylehelp();
 				exit(EXIT_FAILURE);
 			}
 		} else if ((strcmp(argv[currentarg], "--dbdir")) == 0) {
@@ -301,13 +289,14 @@ void parseargs(PARAMS *p, int argc, char **argv)
 			}
 		} else if (strcmp(argv[currentarg], "--oneline") == 0) {
 			cfg.qmode = 9;
-			if (currentarg + 1 < argc && argv[currentarg + 1][0] != '-') {
+			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
 				if (argv[currentarg + 1][0] == 'b') {
 					cfg.ostyle = 4;
 					currentarg++;
 				} else {
-					printf("Error: Invalid mode parameter \"%s\" for --oneline.\n", argv[currentarg + 1]);
-					printf(" Valid parameters:\n");
+					if (!ishelprequest(argv[currentarg + 1]))
+						printf("Error: Invalid mode parameter \"%s\".\n", argv[currentarg + 1]);
+					printf(" Valid parameters for --oneline:\n");
 					printf("    (none) - automatically scaled units visible\n");
 					printf("    b      - all values in bytes\n");
 					exit(EXIT_FAILURE);
@@ -315,11 +304,12 @@ void parseargs(PARAMS *p, int argc, char **argv)
 			}
 		} else if (strcmp(argv[currentarg], "--xml") == 0) {
 			cfg.qmode = 8;
-			if (currentarg + 1 < argc && argv[currentarg + 1][0] != '-' && !isdigit(argv[currentarg + 1][0])) {
+			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
 				p->xmlmode = argv[currentarg + 1][0];
 				if (strlen(argv[currentarg + 1]) != 1 || strchr("afhdmyt", p->xmlmode) == NULL) {
-					printf("Error: Invalid mode parameter \"%s\" for --xml.\n", argv[currentarg + 1]);
-					printf(" Valid parameters:\n");
+					if (!ishelprequest(argv[currentarg + 1]))
+						printf("Error: Invalid mode parameter \"%s\".\n", argv[currentarg + 1]);
+					printf(" Valid parameters for --xml:\n");
 					printf("    a - all (default)\n");
 					printf("    f - only 5 minutes\n");
 					printf("    h - only hours\n");
@@ -341,11 +331,12 @@ void parseargs(PARAMS *p, int argc, char **argv)
 			}
 		} else if (strcmp(argv[currentarg], "--json") == 0) {
 			cfg.qmode = 10;
-			if (currentarg + 1 < argc && argv[currentarg + 1][0] != '-' && !isdigit(argv[currentarg + 1][0])) {
+			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
 				p->jsonmode = argv[currentarg + 1][0];
 				if (strlen(argv[currentarg + 1]) != 1 || strchr("afhdmyt", p->jsonmode) == NULL) {
-					printf("Error: Invalid mode parameter \"%s\" for --json.\n", argv[currentarg + 1]);
-					printf(" Valid parameters:\n");
+					if (!ishelprequest(argv[currentarg + 1]))
+						printf("Error: Invalid mode parameter \"%s\".\n", argv[currentarg + 1]);
+					printf(" Valid parameters for --json:\n");
 					printf("    a - all (default)\n");
 					printf("    f - only 5 minutes\n");
 					printf("    h - only hours\n");
@@ -366,15 +357,16 @@ void parseargs(PARAMS *p, int argc, char **argv)
 				currentarg++;
 			}
 		} else if ((strcmp(argv[currentarg], "-ru") == 0) || (strcmp(argv[currentarg], "--rateunit")) == 0) {
-			if (currentarg + 1 < argc && isdigit(argv[currentarg + 1][0])) {
-				cfg.rateunit = atoi(argv[currentarg + 1]);
-				if (cfg.rateunit > 1 || cfg.rateunit < 0) {
-					printf("Error: Invalid parameter \"%d\" for --rateunit.\n", cfg.rateunit);
-					printf(" Valid parameters:\n");
+			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
+				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 1 || atoi(argv[currentarg + 1]) < 0) {
+					if (!ishelprequest(argv[currentarg + 1]))
+						printf("Error: Invalid parameter \"%s\".\n", argv[currentarg + 1]);
+					printf(" Valid parameters for %s:\n", argv[currentarg]);
 					printf("    0 - bytes\n");
 					printf("    1 - bits\n");
 					exit(EXIT_FAILURE);
 				}
+				cfg.rateunit = atoi(argv[currentarg + 1]);
 				if (debug)
 					printf("Rateunit changed: %d\n", cfg.rateunit);
 				currentarg++;
@@ -391,10 +383,11 @@ void parseargs(PARAMS *p, int argc, char **argv)
 			p->traffic = 1;
 			p->query = 0;
 		} else if ((strcmp(argv[currentarg], "-l") == 0) || (strcmp(argv[currentarg], "--live") == 0)) {
-			if (currentarg + 1 < argc && argv[currentarg + 1][0] != '-') {
+			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
 				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 1 || atoi(argv[currentarg + 1]) < 0) {
-					printf("Error: Invalid mode parameter \"%s\" for -l / --live.\n", argv[currentarg + 1]);
-					printf(" Valid parameters:\n");
+					if (!ishelprequest(argv[currentarg + 1]))
+						printf("Error: Invalid mode parameter \"%s\".\n", argv[currentarg + 1]);
+					printf(" Valid parameters for %s:\n", argv[currentarg]);
 					printf("    0 - show packets per second (default)\n");
 					printf("    1 - show transfer counters\n");
 					exit(EXIT_FAILURE);
@@ -450,10 +443,11 @@ void parseargs(PARAMS *p, int argc, char **argv)
 			}
 		} else if (strcmp(argv[currentarg], "--iflist") == 0) {
 			p->query = 0;
-			if (currentarg + 1 < argc && argv[currentarg + 1][0] != '-') {
+			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
 				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 1 || atoi(argv[currentarg + 1]) < 0) {
-					printf("Error: Invalid mode parameter \"%s\" for --iflist.\n", argv[currentarg + 1]);
-					printf(" Valid parameters:\n");
+					if (!ishelprequest(argv[currentarg + 1]))
+						printf("Error: Invalid mode parameter \"%s\".\n", argv[currentarg + 1]);
+					printf(" Valid parameters for --iflist:\n");
 					printf("    0 - show verbose (default)\n");
 					printf("    1 - show one interface per line\n");
 					exit(EXIT_FAILURE);
@@ -465,10 +459,11 @@ void parseargs(PARAMS *p, int argc, char **argv)
 			exit(EXIT_SUCCESS);
 		} else if (strcmp(argv[currentarg], "--dbiflist") == 0) {
 			p->query = 0;
-			if (currentarg + 1 < argc && argv[currentarg + 1][0] != '-') {
+			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
 				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 1 || atoi(argv[currentarg + 1]) < 0) {
-					printf("Error: Invalid mode parameter \"%s\" for --dbiflist.\n", argv[currentarg + 1]);
-					printf(" Valid parameters:\n");
+					if (!ishelprequest(argv[currentarg + 1]))
+						printf("Error: Invalid mode parameter \"%s\".\n", argv[currentarg + 1]);
+					printf(" Valid parameters for --dbiflist:\n");
 					printf("    0 - show verbose (default)\n");
 					printf("    1 - show one interface per line\n");
 					exit(EXIT_FAILURE);
@@ -494,8 +489,23 @@ void parseargs(PARAMS *p, int argc, char **argv)
 			printf("vnStat %s by Teemu Toivola <tst at iki dot fi>\n", getversion());
 			exit(EXIT_SUCCESS);
 		} else {
-			printf("Unknown parameter \"%s\". Use --help for help.\n", argv[currentarg]);
-			exit(EXIT_FAILURE);
+			if (argv[currentarg][0] == '-' || strlen(argv[currentarg]) == 1) {
+				printf("Unknown parameter \"%s\". Use --help for help.\n", argv[currentarg]);
+				exit(EXIT_FAILURE);
+			} else {
+				if (strlen(argv[currentarg]) > 31) {
+					printf("Error: Interface name is limited to 31 characters.\n");
+					exit(EXIT_FAILURE);
+				}
+				strncpy_nt(p->interface, argv[currentarg], 32);
+				if (strlen(p->interface)) {
+					p->defaultiface = 0;
+				} else {
+					strncpy_nt(p->definterface, p->interface, 32);
+				}
+				if (debug)
+					printf("Used interface: \"%s\"\n", p->interface);
+			}
 		}
 	}
 
@@ -504,6 +514,17 @@ void parseargs(PARAMS *p, int argc, char **argv)
 	}
 }
 
+void showstylehelp(void)
+{
+	printf(" Valid parameters for --style.:\n");
+	printf("    0 - a more narrow output\n");
+	printf("    1 - enable bar column if available\n");
+	printf("    2 - average traffic rate in summary output\n");
+	printf("    3 - average traffic rate in all outputs if available\n");
+	printf("    4 - disable terminal control characters in -l / --live\n");
+	printf("        and show raw values in --oneline\n");
+}
+
 void handleremoveinterface(PARAMS *p)
 {
 	if (!p->removeiface) {
@@ -521,8 +542,8 @@ void handleremoveinterface(PARAMS *p)
 	}
 
 	if (!p->force) {
-		printf("Warning:\nThe current option would remove all data\nabout interface \"%s\" from the database.\n", p->interface);
-		printf("Use --force in order to really do that.\n");
+		printf("Warning:\nThe current option would remove all data about interface \"%s\" from the database. ", p->interface);
+		printf("Add --force in order to really do that.\n");
 		exit(EXIT_FAILURE);
 	}
 
@@ -535,8 +556,7 @@ void handleremoveinterface(PARAMS *p)
 
 	if (db_removeinterface(p->interface)) {
 		printf("Interface \"%s\" removed from database.\n", p->interface);
-		printf("The interface will no longer be monitored. Use --add\n");
-		printf("if monitoring the interface is again needed.\n");
+		printf("The interface will no longer be monitored. Use --add if monitoring the interface is again needed.\n");
 #ifndef CHECK_VNSTAT
 		db_close();
 		exit(EXIT_SUCCESS);
@@ -574,8 +594,8 @@ void handlerenameinterface(PARAMS *p)
 	}
 
 	if (!p->force) {
-		printf("Warning:\nThe current option would rename\ninterface \"%s\" -> \"%s\" in the database.\n", p->interface, p->newifname);
-		printf("Use --force in order to really do that.\n");
+		printf("Warning:\nThe current option would rename interface \"%s\" -> \"%s\" in the database. ", p->interface, p->newifname);
+		printf("Add --force in order to really do that.\n");
 		exit(EXIT_FAILURE);
 	}
 
@@ -638,9 +658,13 @@ void handleaddinterface(PARAMS *p)
 	}
 #endif
 
-	printf("Adding interface \"%s\" for monitoring to database...\n", p->interface);
+	printf("Adding interface \"%s\" to database for monitoring.\n", p->interface);
 	if (db_addinterface(p->interface)) {
-		printf("\nRestart the vnStat daemon if it is currently running in order to start monitoring \"%s\".\n", p->interface);
+		if (cfg.rescanonsave) {
+			printf("vnStat daemon will automatically start monitoring \"%s\" within %d minutes if the daemon process is currently running.\n", p->interface, cfg.saveinterval);
+		} else {
+			printf("Restart vnStat daemon if it is currently running in order to start monitoring \"%s\".\n", p->interface);
+		}
 		handlesetalias(p);
 #ifndef CHECK_VNSTAT
 		db_close();
@@ -809,8 +833,8 @@ void handletrafficmeters(PARAMS *p)
 		if (p->defaultiface) {
 			printf("Error: Configured default interface \"%s\" isn't available.\n\n", p->interface);
 			if (strlen(cfg.cfgfile)) {
-				printf("Update \"Interface\" keyword value in \"%s\" to change\n", cfg.cfgfile);
-				printf("the default interface or give an alternative interface\nusing the -i parameter.\n\n");
+				printf("Update \"Interface\" keyword value in configuration file \"%s\" to change ", cfg.cfgfile);
+				printf("the default interface or give an alternative interface using the -i parameter.\n\n");
 			} else {
 				printf("An alternative interface can be given using the -i parameter.\n\n");
 			}
