@@ -21,6 +21,7 @@ vnStat daemon - Copyright (C) 2008-2021 Teemu Toivola <tst@iki.fi>
 #include "cfg.h"
 #include "ibw.h"
 #include "id.h"
+#include "misc.h"
 #include "daemon.h"
 #include "vnstatd.h"
 
@@ -56,7 +57,7 @@ int main(int argc, char *argv[])
 	timeused_debug("daemon_startup", 1);
 
 	/* load config if available */
-	if (!loadcfg(s.cfgfile)) {
+	if (!loadcfg(s.cfgfile, CT_Daemon)) {
 		return 1;
 	}
 	if (!ibwloadcfg(s.cfgfile)) {
@@ -121,7 +122,7 @@ int main(int argc, char *argv[])
 	timeused_debug("daemon_startup", 0);
 	s.running = 1;
 
-#if defined(__linux__)
+#if defined(__linux__) && HAVE_LINUX_RTNETLINK_H
 #if HAVE_DECL_IFLA_STATS64
 	snprintf(errorstring, 1024, "vnStat daemon %s started. (pid:%d uid:%d gid:%d 64-bit)", getversion(), (int)getpid(), (int)getuid(), (int)getgid());
 #else
@@ -161,7 +162,7 @@ int main(int argc, char *argv[])
 		if (s.dbifcount != 0) {
 			previflisthash = s.iflisthash;
 			interfacechangecheck(&s);
-			if (s.alwaysadd && s.iflisthash != previflisthash && previflisthash != 0) {
+			if (cfg.alwaysadd && s.iflisthash != previflisthash && previflisthash != 0) {
 				temp = s.dbifcount;
 				s.dbifcount += addinterfaces(&s);
 				if (temp != s.dbifcount) {
@@ -246,7 +247,7 @@ void showhelp(void)
 	printf("      -g, --group <group>      set daemon process group\n");
 	printf("      --config <config file>   select used config file\n");
 	printf("      --noadd                  prevent startup if database has no interfaces\n");
-	printf("      --alwaysadd              automatically start monitoring all new interfaces\n");
+	printf("      --alwaysadd [mode]       automatically start monitoring all new interfaces\n");
 	printf("      --initdb                 create empty database and exit\n\n");
 
 	printf("See also \"man vnstatd\".\n");
@@ -297,7 +298,27 @@ void parseargs(DSTATE *s, int argc, char **argv)
 		} else if (strcmp(argv[currentarg], "--noadd") == 0) {
 			s->noadd = 1;
 		} else if (strcmp(argv[currentarg], "--alwaysadd") == 0) {
-			s->alwaysadd = 1;
+			if (currentarg + 1 < argc && (strlen(argv[currentarg + 1]) == 1 || ishelprequest(argv[currentarg + 1]))) {
+				if (!isdigit(argv[currentarg + 1][0]) || atoi(argv[currentarg + 1]) > 1 || atoi(argv[currentarg + 1]) < 0) {
+					if (!ishelprequest(argv[currentarg + 1]))
+						printf("Error: Invalid mode parameter \"%s\".\n", argv[currentarg + 1]);
+					printf(" Valid parameters for %s:\n", argv[currentarg]);
+					printf("    0 - disabled");
+					if (!cfg.alwaysadd) {
+						printf(" (default)");
+					}
+					printf("\n    1 - enabled");
+					if (cfg.alwaysadd) {
+						printf(" (default)");
+					}
+					printf("\n No mode parameter results in feature being enabled.\n");
+					exit(EXIT_FAILURE);
+				}
+				cfg.alwaysadd = atoi(argv[currentarg + 1]);
+				currentarg++;
+			} else {
+				cfg.alwaysadd = 1;
+			}
 		} else if (strcmp(argv[currentarg], "--initdb") == 0) {
 			s->initdb = 1;
 			s->showhelp = 0;
@@ -322,9 +343,9 @@ void parseargs(DSTATE *s, int argc, char **argv)
 		}
 	}
 
-	if (s->noadd && s->alwaysadd) {
-		printf("Error: --noadd and --alwaysadd can't both be used at the same time.\n");
-		exit(EXIT_FAILURE);
+	if (s->noadd && cfg.alwaysadd) {
+		printf("Warning: --noadd and --alwaysadd can't both be enabled at the same time. --alwaysadd has been disabled.\n");
+		cfg.alwaysadd = 0;
 	}
 
 	if (s->rundaemon && debug) {
