@@ -9,7 +9,6 @@ START_TEST(db_close_does_no_harm_when_db_is_already_closed)
 {
 	int ret;
 
-	defaultcfg();
 
 	ret = db_close();
 	ck_assert_int_eq(ret, 1);
@@ -20,7 +19,6 @@ START_TEST(db_open_rw_can_create_database_if_file_does_not_exist)
 {
 	int ret;
 
-	defaultcfg();
 
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
@@ -33,7 +31,6 @@ START_TEST(db_open_ro_cannot_create_a_database)
 {
 	int ret;
 
-	defaultcfg();
 	suppress_output();
 
 	ret = db_open_ro();
@@ -43,7 +40,6 @@ END_TEST
 
 START_TEST(db_getinfo_fails_with_no_open_db)
 {
-	defaultcfg();
 	suppress_output();
 
 	ck_assert_int_eq(strlen(db_getinfo("foofoo")), 0);
@@ -54,8 +50,6 @@ START_TEST(db_getinfo_fails_with_nonexisting_name)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -70,8 +64,6 @@ START_TEST(db_getinfo_can_get_dbversion)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -84,7 +76,6 @@ END_TEST
 
 START_TEST(db_setinfo_fails_with_no_open_db)
 {
-	defaultcfg();
 	suppress_output();
 
 	ck_assert_int_eq(db_setinfo("foo", "bar", 0), 0);
@@ -96,8 +87,6 @@ START_TEST(db_setinfo_can_set_infos)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -113,8 +102,6 @@ START_TEST(db_setinfo_can_update_infos)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -136,8 +123,6 @@ START_TEST(db_setinfo_can_not_update_nonexisting_name)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -153,7 +138,6 @@ END_TEST
 
 START_TEST(db_addtraffic_with_no_traffic_does_nothing)
 {
-	defaultcfg();
 	suppress_output();
 
 	ck_assert_int_eq(db_addtraffic("eth0", 0, 0), 0);
@@ -164,7 +148,24 @@ START_TEST(db_addtraffic_can_add_traffic_and_interfaces)
 {
 	int ret;
 
-	defaultcfg();
+	ret = db_open_rw(1);
+	ck_assert_int_eq(ret, 1);
+
+	ck_assert_int_eq(db_addtraffic("eth0", 0, 0), 1);
+	ck_assert_int_eq(db_addtraffic("eth0", 12, 34), 1);
+	ck_assert_int_eq(db_addtraffic("eth1", 56, 78), 1);
+
+	ret = db_close();
+	ck_assert_int_eq(ret, 1);
+}
+END_TEST
+
+START_TEST(db_addtraffic_can_add_traffic_and_interfaces_utc)
+{
+	int ret;
+
+	cfg.useutc = 1;
+	validatecfg(CT_Daemon);
 
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
@@ -183,8 +184,6 @@ START_TEST(db_addtraffic_dated_does_not_touch_updated_time)
 	int ret;
 	interfaceinfo info;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -238,7 +237,6 @@ END_TEST
 
 START_TEST(db_setactive_fails_with_no_open_db)
 {
-	defaultcfg();
 	suppress_output();
 
 	ck_assert_int_eq(db_setactive("eth0", 0), 0);
@@ -250,8 +248,6 @@ START_TEST(db_setactive_fails_if_interface_does_not_exist_in_database)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -267,8 +263,6 @@ START_TEST(db_setactive_can_change_interface_activity_status)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -287,7 +281,6 @@ END_TEST
 
 START_TEST(db_setalias_fails_with_no_open_db)
 {
-	defaultcfg();
 	suppress_output();
 
 	ck_assert_int_eq(db_setalias("eth0", "The Internet"), 0);
@@ -298,8 +291,6 @@ START_TEST(db_setalias_fails_if_interface_does_not_exist_in_database)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -314,8 +305,6 @@ START_TEST(db_setalias_can_change_interface_alias)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -332,7 +321,6 @@ END_TEST
 
 START_TEST(db_setupdated_fails_with_no_open_db)
 {
-	defaultcfg();
 	suppress_output();
 
 	ck_assert_int_eq(db_setupdated("eth0", 123456), 0);
@@ -343,8 +331,6 @@ START_TEST(db_setupdated_fails_if_interface_does_not_exist_in_database)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -359,8 +345,6 @@ START_TEST(db_setupdated_can_change_updated)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -376,7 +360,6 @@ START_TEST(db_addinterface_fails_with_no_open_db)
 {
 	int ret;
 
-	defaultcfg();
 	suppress_output();
 
 	ret = db_addinterface("eth0");
@@ -388,8 +371,6 @@ START_TEST(db_addinterface_can_add_interfaces)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -407,7 +388,6 @@ START_TEST(db_addinterface_can_not_add_same_interface_twice)
 {
 	int ret;
 
-	defaultcfg();
 	suppress_output();
 
 	ret = db_open_rw(1);
@@ -431,8 +411,6 @@ START_TEST(db_removeinterface_knows_if_interface_exists)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -452,8 +430,6 @@ START_TEST(db_removeinterface_can_remove_interfaces)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -492,8 +468,6 @@ START_TEST(db_renameinterface_knows_if_interface_exists)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -513,8 +487,6 @@ START_TEST(db_renameinterface_can_rename_interfaces)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -559,7 +531,6 @@ START_TEST(db_getinterfacecount_counts_interfaces)
 {
 	int ret;
 
-	defaultcfg();
 	suppress_output();
 
 	ret = db_open_rw(1);
@@ -595,8 +566,6 @@ START_TEST(db_getinterfacecountbyname_counts_interfaces)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 
@@ -643,8 +612,6 @@ START_TEST(db_getcounters_with_no_interface)
 	int ret;
 	uint64_t rx, tx;
 
-	defaultcfg();
-
 	rx = tx = 1;
 
 	ret = db_open_rw(1);
@@ -665,8 +632,6 @@ START_TEST(db_setcounters_with_no_interface)
 	int ret;
 	uint64_t rx, tx;
 
-	defaultcfg();
-
 	rx = tx = 1;
 
 	ret = db_open_rw(1);
@@ -691,8 +656,6 @@ START_TEST(db_interface_info_manipulation)
 	uint64_t rx, tx, c;
 	interfaceinfo info;
 
-	defaultcfg();
-
 	rx = tx = 1;
 
 	ret = db_open_rw(1);
@@ -765,8 +728,6 @@ START_TEST(db_getiflist_lists_interfaces)
 	int ret;
 	iflist *dbifl = NULL, *dbifl_i = NULL;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 	ret = db_addinterface("a");
@@ -810,8 +771,6 @@ START_TEST(db_maintenance_does_not_fault)
 {
 	int ret;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 	ret = db_addinterface("eth0");
@@ -835,8 +794,6 @@ START_TEST(db_data_can_be_inserted)
 	int ret;
 	interfaceinfo info;
 
-	defaultcfg();
-
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
 	ret = db_addinterface("eth0");
@@ -884,7 +841,107 @@ START_TEST(db_data_can_be_retrieved)
 	dbdatalist *datalist = NULL, *datalist_iterator = NULL;
 	dbdatalistinfo datainfo;
 
-	defaultcfg();
+	ret = db_open_rw(1);
+	ck_assert_int_eq(ret, 1);
+	ret = db_addinterface("eth0");
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_insertdata("hour", "eth0", 1, 2, 3);
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_insertdata("hour", "eth0", 10, 20, 10000);
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_getdata(&datalist, &datainfo, "eth0", "hour", 2);
+	ck_assert_int_eq(ret, 1);
+
+	ck_assert_int_eq(datainfo.count, 2);
+	ck_assert_int_eq(datainfo.minrx, 1);
+	ck_assert_int_eq(datainfo.maxrx, 10);
+	ck_assert_int_eq(datainfo.sumrx, 11);
+	ck_assert_int_eq(datainfo.mintx, 2);
+	ck_assert_int_eq(datainfo.maxtx, 20);
+	ck_assert_int_eq(datainfo.sumtx, 22);
+	/* db_insertdata rounds the timestamps to full hours */
+	ck_assert_int_eq((int)datainfo.maxtime, 7200);
+	ck_assert_int_eq((int)datainfo.mintime, 0);
+
+	datalist_iterator = datalist;
+
+	ck_assert_int_eq(datalist_iterator->rx, 1);
+	ck_assert_int_eq(datalist_iterator->tx, 2);
+	ck_assert_int_eq(datalist_iterator->timestamp, 0);
+
+	datalist_iterator = datalist_iterator->next;
+
+	ck_assert_int_eq(datalist_iterator->rx, 10);
+	ck_assert_int_eq(datalist_iterator->tx, 20);
+	ck_assert_int_eq(datalist_iterator->timestamp, 7200);
+
+	dbdatalistfree(&datalist);
+
+	ret = db_close();
+	ck_assert_int_eq(ret, 1);
+}
+END_TEST
+
+START_TEST(db_data_can_be_inserted_utc)
+{
+	int ret;
+	interfaceinfo info;
+
+	cfg.useutc = 1;
+	validatecfg(CT_Daemon);
+
+	ret = db_open_rw(1);
+	ck_assert_int_eq(ret, 1);
+	ret = db_addinterface("eth0");
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_insertdata("foo", "eth0", 1, 2, 3);
+	ck_assert_int_eq(ret, 0);
+
+	ret = db_insertdata("hour", "eth1", 1, 2, 3);
+	ck_assert_int_eq(ret, 0);
+
+	ret = db_insertdata("hour", "eth0", 1, 2, 3);
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_insertdata("day", "eth0", 1, 2, 3);
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_insertdata("month", "eth0", 1, 2, 3);
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_insertdata("year", "eth0", 1, 2, 3);
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_insertdata("top", "eth0", 1, 2, 3);
+	ck_assert_int_eq(ret, 1);
+
+	/* verify that totals don't get changed */
+	ret = db_getinterfaceinfo("eth0", &info);
+	ck_assert_int_eq(ret, 1);
+	ck_assert_int_eq(info.active, 1);
+	ck_assert_int_eq(info.rxcounter, 0);
+	ck_assert_int_eq(info.txcounter, 0);
+	ck_assert_int_eq(info.rxtotal, 0);
+	ck_assert_int_eq(info.txtotal, 0);
+	ck_assert_int_ne(info.created, 0);
+
+	ret = db_close();
+	ck_assert_int_eq(ret, 1);
+}
+END_TEST
+
+START_TEST(db_data_can_be_retrieved_utc)
+{
+	int ret;
+	dbdatalist *datalist = NULL, *datalist_iterator = NULL;
+	dbdatalistinfo datainfo;
+
+	cfg.useutc = 1;
+	validatecfg(CT_Daemon);
 
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
@@ -948,7 +1005,6 @@ END_TEST
 START_TEST(db_validate_with_valid_version)
 {
 	int ret;
-	defaultcfg();
 	suppress_output();
 
 	ret = db_open_rw(1);
@@ -970,7 +1026,6 @@ END_TEST
 START_TEST(db_validate_with_no_version)
 {
 	int ret;
-	defaultcfg();
 	suppress_output();
 
 	ret = db_open_rw(1);
@@ -995,7 +1050,6 @@ END_TEST
 START_TEST(db_validate_with_low_version)
 {
 	int ret;
-	defaultcfg();
 	suppress_output();
 
 	ret = db_open_rw(1);
@@ -1020,7 +1074,6 @@ END_TEST
 START_TEST(db_validate_with_high_version)
 {
 	int ret;
-	defaultcfg();
 	suppress_output();
 
 	ret = db_open_rw(1);
@@ -1045,7 +1098,6 @@ END_TEST
 void range_test_month_setup(void)
 {
 	int ret, i;
-	defaultcfg();
 	cfg.monthrotate = 2; /* this should have no effect */
 
 	ret = db_open_rw(1);
@@ -1408,7 +1460,6 @@ END_TEST
 void range_test_hour_setup(void)
 {
 	int ret, i, j;
-	defaultcfg();
 	cfg.monthrotate = 2; /* this should have no effect */
 
 	ret = db_open_rw(1);
@@ -1746,7 +1797,6 @@ START_TEST(db_getdata_range_with_merged_interfaces)
 	dbdatalist *datalist = NULL, *datalist_i = NULL;
 	dbdatalistinfo datainfo;
 
-	defaultcfg();
 	cfg.monthrotate = 1;
 
 	ret = db_open_rw(1);
@@ -1794,7 +1844,6 @@ START_TEST(db_addtraffic_without_monthrotate)
 	dbdatalist *datalist = NULL, *datalist_i = NULL;
 	dbdatalistinfo datainfo;
 
-	defaultcfg();
 	cfg.monthrotate = 1;
 
 	ret = db_open_rw(1);
@@ -1838,7 +1887,6 @@ START_TEST(db_addtraffic_with_monthrotate)
 	dbdatalist *datalist = NULL, *datalist_i = NULL;
 	dbdatalistinfo datainfo;
 
-	defaultcfg();
 	cfg.monthrotate = 7;
 
 	ret = db_open_rw(1);
@@ -1881,9 +1929,104 @@ START_TEST(db_addtraffic_with_monthrotate)
 }
 END_TEST
 
+START_TEST(db_addtraffic_without_monthrotate_utc)
+{
+	int ret, i;
+	char timestamp[64];
+	dbdatalist *datalist = NULL, *datalist_i = NULL;
+	dbdatalistinfo datainfo;
+
+	cfg.monthrotate = 1;
+	cfg.useutc = 1;
+	validatecfg(CT_Daemon);
+
+	ret = db_open_rw(1);
+	ck_assert_int_eq(ret, 1);
+	ret = db_addinterface("ethtest");
+	ck_assert_int_eq(ret, 1);
+
+	for (i = 1; i <= 20; i++) {
+		ret = db_addtraffic_dated("ethtest", 1, 2, get_timestamp(2000, 2, i, 0, 0));
+		ck_assert_int_eq(ret, 1);
+	}
+
+	ret = db_getdata_range(&datalist, &datainfo, "ethtest", "month", 0, "", "");
+	ck_assert_int_eq(ret, 1);
+	ck_assert_int_eq(datainfo.count, 1);
+	datalist_i = datalist;
+	i = 0;
+	while (datalist_i != NULL) {
+		switch (i) {
+			case 0:
+				strftime(timestamp, 64, "%Y-%m-%d", localtime(&datalist_i->timestamp));
+				ck_assert_str_eq(timestamp, "2000-02-01");
+				ck_assert_int_eq(datalist_i->rx, 20);
+				ck_assert_int_eq(datalist_i->tx, 40);
+				break;
+		}
+		datalist_i = datalist_i->next;
+		i++;
+	}
+	dbdatalistfree(&datalist);
+
+	ret = db_close();
+	ck_assert_int_eq(ret, 1);
+}
+END_TEST
+
+START_TEST(db_addtraffic_with_monthrotate_utc)
+{
+	int ret, i;
+	char timestamp[64];
+	dbdatalist *datalist = NULL, *datalist_i = NULL;
+	dbdatalistinfo datainfo;
+
+	cfg.monthrotate = 7;
+	cfg.useutc = 1;
+	validatecfg(CT_Daemon);
+
+	ret = db_open_rw(1);
+	ck_assert_int_eq(ret, 1);
+	ret = db_addinterface("ethtest");
+	ck_assert_int_eq(ret, 1);
+
+	for (i = 1; i <= 20; i++) {
+		ret = db_addtraffic_dated("ethtest", 1, 2, get_timestamp(2000, 2, i, 0, 0));
+		ck_assert_int_eq(ret, 1);
+	}
+
+	ret = db_getdata_range(&datalist, &datainfo, "ethtest", "month", 0, "", "");
+	ck_assert_int_eq(ret, 1);
+	ck_assert_int_eq(datainfo.count, 2);
+	datalist_i = datalist;
+	i = 0;
+	while (datalist_i != NULL) {
+		switch (i) {
+			case 0:
+				strftime(timestamp, 64, "%Y-%m-%d", localtime(&datalist_i->timestamp));
+				ck_assert_str_eq(timestamp, "2000-01-01");
+				ck_assert_int_eq(datalist_i->rx, 6);
+				ck_assert_int_eq(datalist_i->tx, 12);
+				break;
+			case 1:
+				strftime(timestamp, 64, "%Y-%m-%d", localtime(&datalist_i->timestamp));
+				ck_assert_str_eq(timestamp, "2000-02-01");
+				ck_assert_int_eq(datalist_i->rx, 14);
+				ck_assert_int_eq(datalist_i->tx, 28);
+				break;
+		}
+		datalist_i = datalist_i->next;
+		i++;
+	}
+	dbdatalistfree(&datalist);
+
+	ret = db_close();
+	ck_assert_int_eq(ret, 1);
+}
+END_TEST
+
 START_TEST(db_get_date_generator_can_generate_dates)
 {
-	defaultcfg();
 
 	ck_assert_ptr_ne(strstr(db_get_date_generator(0, 0, "foo"), "minutes"), NULL);
 	ck_assert_ptr_ne(strstr(db_get_date_generator(1, 0, "foo"), "strftime('%Y-%m-%d %H:00:00', foo, 'localtime')"), NULL);
@@ -1903,7 +2046,6 @@ END_TEST
 
 START_TEST(db_get_date_generator_can_generate_dates_with_monthrotate)
 {
-	defaultcfg();
 
 	cfg.monthrotate = 10;
 	cfg.monthrotateyears = 0;
@@ -1941,6 +2083,68 @@ START_TEST(db_get_date_generator_can_generate_dates_with_monthrotate)
 }
 END_TEST
 
+START_TEST(db_get_date_generator_can_generate_dates_utc)
+{
+	cfg.useutc = 1;
+	validatecfg(CT_Daemon);
+
+	ck_assert_ptr_ne(strstr(db_get_date_generator(0, 0, "foo"), "minutes"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(1, 0, "foo"), "strftime('%Y-%m-%d %H:00:00', foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(2, 0, "foo"), "date(foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(5, 0, "foo"), "date(foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(3, 0, "foo"), "strftime('%Y-%m-01', foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(4, 0, "foo"), "strftime('%Y-01-01', foo)"), NULL);
+
+	ck_assert_ptr_ne(strstr(db_get_date_generator(0, 1, "foo"), "minutes"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(1, 1, "foo"), "strftime('%Y-%m-%d %H:00:00', foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(2, 1, "foo"), "date(foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(5, 1, "foo"), "date(foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(3, 1, "foo"), "strftime('%Y-%m-01', foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(4, 1, "foo"), "strftime('%Y-01-01', foo)"), NULL);
+}
+END_TEST
+
+START_TEST(db_get_date_generator_can_generate_dates_with_monthrotate_utc)
+{
+	cfg.useutc = 1;
+	validatecfg(CT_Daemon);
+
+	cfg.monthrotate = 10;
+	cfg.monthrotateyears = 0;
+
+	ck_assert_ptr_ne(strstr(db_get_date_generator(0, 0, "foo"), "minutes"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(1, 0, "foo"), "strftime('%Y-%m-%d %H:00:00', foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(2, 0, "foo"), "date(foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(5, 0, "foo"), "date(foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(3, 0, "foo"), "strftime('%Y-%m-01', datetime(foo, '-9 days'))"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(4, 0, "foo"), "strftime('%Y-01-01', foo)"), NULL);
+
+	ck_assert_ptr_ne(strstr(db_get_date_generator(0, 1, "foo"), "minutes"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(1, 1, "foo"), "strftime('%Y-%m-%d %H:00:00', foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(2, 1, "foo"), "date(foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(5, 1, "foo"), "date(foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(3, 1, "foo"), "strftime('%Y-%m-01', foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(4, 1, "foo"), "strftime('%Y-01-01', foo)"), NULL);
+
+	cfg.monthrotate = 8;
+	cfg.monthrotateyears = 1;
+
+	ck_assert_ptr_ne(strstr(db_get_date_generator(0, 0, "foo"), "minutes"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(1, 0, "foo"), "strftime('%Y-%m-%d %H:00:00', foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(2, 0, "foo"), "date(foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(5, 0, "foo"), "date(foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(3, 0, "foo"), "strftime('%Y-%m-01', datetime(foo, '-7 days'))"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(4, 0, "foo"), "strftime('%Y-01-01', datetime(foo, '-7 days'))"), NULL);
+
+	ck_assert_ptr_ne(strstr(db_get_date_generator(0, 1, "foo"), "minutes"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(1, 1, "foo"), "strftime('%Y-%m-%d %H:00:00', foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(2, 1, "foo"), "date(foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(5, 1, "foo"), "date(foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(3, 1, "foo"), "strftime('%Y-%m-01', foo)"), NULL);
+	ck_assert_ptr_ne(strstr(db_get_date_generator(4, 1, "foo"), "strftime('%Y-01-01', foo)"), NULL);
+}
+END_TEST
+
 START_TEST(getifaceinquery_does_not_mess_regular_interfaces)
 {
 	char *result;
@@ -2023,7 +2227,6 @@ START_TEST(db_getinterfaceid_can_get_ids)
 {
 	int ret;
 
-	defaultcfg();
 
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
@@ -2074,7 +2277,6 @@ START_TEST(db_getinterfaceidin_can_get_in_groups)
 	int ret;
 	char *result;
 
-	defaultcfg();
 
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
@@ -2118,7 +2320,6 @@ START_TEST(db_getinterfaceidin_can_handle_error_situations)
 	int ret;
 	char *result;
 
-	defaultcfg();
 
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
@@ -2142,7 +2343,6 @@ START_TEST(db_getinterfaceinfo_can_handle_interface_merges)
 	int ret;
 	interfaceinfo info;
 
-	defaultcfg();
 
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
@@ -2194,7 +2394,6 @@ START_TEST(db_getinterfaceinfo_can_handle_invalid_input)
 	int ret;
 	interfaceinfo info;
 
-	defaultcfg();
 
 	ret = db_open_rw(1);
 	ck_assert_int_eq(ret, 1);
@@ -2249,6 +2448,104 @@ START_TEST(getqueryinterfacecount_can_count)
 }
 END_TEST
 
+START_TEST(top_list_returns_items_in_correct_order)
+{
+	int ret;
+	dbdatalist *datalist = NULL, *datalist_iterator = NULL;
+	dbdatalistinfo datainfo;
+	uint64_t previous_entry;
+
+	ret = db_open_rw(1);
+	ck_assert_int_eq(ret, 1);
+	ret = db_addinterface("ethtest");
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_addtraffic_dated("ethtest", 0, 0, get_timestamp(2000, 3, 10, 0, 0));
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_addtraffic_dated("ethtest", 12, 34, get_timestamp(2000, 3, 11, 0, 0));
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_addtraffic_dated("ethtest", 0, 0, get_timestamp(2000, 3, 12, 0, 0));
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_addtraffic_dated("ethtest", 0, 0, get_timestamp(2000, 3, 13, 0, 0));
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_addtraffic_dated("ethtest", 56, 78, get_timestamp(2000, 3, 14, 0, 0));
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_addtraffic_dated("ethtest", 1, 1, get_timestamp(2000, 3, 15, 0, 0));
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_addtraffic_dated("ethtest", 1, 1, get_timestamp(2000, 3, 16, 0, 0));
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_addtraffic_dated("ethtest", 45, 1, get_timestamp(2000, 3, 17, 0, 0));
+	ck_assert_int_eq(ret, 1);
+
+	ret = db_getdata(&datalist, &datainfo, "ethtest", "top", 10);
+	ck_assert_int_eq(ret, 1);
+
+	ck_assert_int_eq(datainfo.count, 8);
+	ck_assert_int_eq(datainfo.maxrx, 56);
+	ck_assert_int_eq(datainfo.maxtx, 78);
+	ck_assert_int_eq(datainfo.minrx, 0);
+	ck_assert_int_eq(datainfo.mintx, 0);
+
+	datalist_iterator = datalist;
+
+	ck_assert_int_eq(datalist_iterator->rx, 56);
+	ck_assert_int_eq(datalist_iterator->tx, 78);
+
+	datalist_iterator = datalist_iterator->next;
+
+	ck_assert_int_eq(datalist_iterator->rx, 12);
+	ck_assert_int_eq(datalist_iterator->tx, 34);
+
+	datalist_iterator = datalist_iterator->next;
+
+	ck_assert_int_eq(datalist_iterator->rx, 45);
+	ck_assert_int_eq(datalist_iterator->tx, 1);
+
+	datalist_iterator = datalist_iterator->next;
+
+	ck_assert_int_eq(datalist_iterator->rx, 1);
+	ck_assert_int_eq(datalist_iterator->tx, 1);
+	previous_entry = (uint64_t)datalist_iterator->timestamp;
+
+	datalist_iterator = datalist_iterator->next;
+
+	ck_assert_int_eq(datalist_iterator->rx, 1);
+	ck_assert_int_eq(datalist_iterator->tx, 1);
+	ck_assert_int_lt(previous_entry, (uint64_t)datalist_iterator->timestamp);
+
+	datalist_iterator = datalist_iterator->next;
+
+	ck_assert_int_eq(datalist_iterator->rx, 0);
+	ck_assert_int_eq(datalist_iterator->tx, 0);
+	previous_entry = (uint64_t)datalist_iterator->timestamp;
+
+	datalist_iterator = datalist_iterator->next;
+
+	ck_assert_int_eq(datalist_iterator->rx, 0);
+	ck_assert_int_eq(datalist_iterator->tx, 0);
+	ck_assert_int_lt(previous_entry, (uint64_t)datalist_iterator->timestamp);
+	previous_entry = (uint64_t)datalist_iterator->timestamp;
+
+	datalist_iterator = datalist_iterator->next;
+
+	ck_assert_int_eq(datalist_iterator->rx, 0);
+	ck_assert_int_eq(datalist_iterator->tx, 0);
+	ck_assert_int_lt(previous_entry, (uint64_t)datalist_iterator->timestamp);
+
+	dbdatalistfree(&datalist);
+
+	ret = db_close();
+	ck_assert_int_eq(ret, 1);
+}
+END_TEST
+
 void add_dbsql_tests(Suite *s)
 {
 	TCase *tc_dbsql = tcase_create("DB SQL");
@@ -2266,6 +2563,7 @@ void add_dbsql_tests(Suite *s)
 	tcase_add_test(tc_dbsql, db_setinfo_can_not_update_nonexisting_name);
 	tcase_add_test(tc_dbsql, db_addtraffic_with_no_traffic_does_nothing);
 	tcase_add_test(tc_dbsql, db_addtraffic_can_add_traffic_and_interfaces);
+	tcase_add_test(tc_dbsql, db_addtraffic_can_add_traffic_and_interfaces_utc);
 	tcase_add_test(tc_dbsql, db_addtraffic_dated_does_not_touch_updated_time);
 	tcase_add_test(tc_dbsql, db_getinterfacecount_counts_interfaces);
 	tcase_add_test(tc_dbsql, db_getinterfacecountbyname_counts_interfaces);
@@ -2292,6 +2590,8 @@ void add_dbsql_tests(Suite *s)
 	tcase_add_test(tc_dbsql, db_maintenance_does_not_fault);
 	tcase_add_test(tc_dbsql, db_data_can_be_inserted);
 	tcase_add_test(tc_dbsql, db_data_can_be_retrieved);
+	tcase_add_test(tc_dbsql, db_data_can_be_inserted_utc);
+	tcase_add_test(tc_dbsql, db_data_can_be_retrieved_utc);
 	tcase_add_test(tc_dbsql, db_fatal_errors_get_detected);
 	tcase_add_test(tc_dbsql, db_validate_with_valid_version);
 	tcase_add_test(tc_dbsql, db_validate_with_no_version);
@@ -2319,8 +2619,12 @@ void add_dbsql_tests(Suite *s)
 	tcase_add_test(tc_dbsql, db_getdata_range_with_merged_interfaces);
 	tcase_add_test(tc_dbsql, db_addtraffic_without_monthrotate);
 	tcase_add_test(tc_dbsql, db_addtraffic_with_monthrotate);
+	tcase_add_test(tc_dbsql, db_addtraffic_without_monthrotate_utc);
+	tcase_add_test(tc_dbsql, db_addtraffic_with_monthrotate_utc);
 	tcase_add_test(tc_dbsql, db_get_date_generator_can_generate_dates);
 	tcase_add_test(tc_dbsql, db_get_date_generator_can_generate_dates_with_monthrotate);
+	tcase_add_test(tc_dbsql, db_get_date_generator_can_generate_dates_utc);
+	tcase_add_test(tc_dbsql, db_get_date_generator_can_generate_dates_with_monthrotate_utc);
 	tcase_add_test(tc_dbsql, getifaceinquery_does_not_mess_regular_interfaces);
 	tcase_add_test(tc_dbsql, getifaceinquery_can_create_merge_queries);
 	tcase_add_test(tc_dbsql, getifaceinquery_does_not_tolerate_nonsense);
@@ -2330,5 +2634,6 @@ void add_dbsql_tests(Suite *s)
 	tcase_add_test(tc_dbsql, db_getinterfaceinfo_can_handle_interface_merges);
 	tcase_add_test(tc_dbsql, db_getinterfaceinfo_can_handle_invalid_input);
 	tcase_add_test(tc_dbsql, getqueryinterfacecount_can_count);
+	tcase_add_test(tc_dbsql, top_list_returns_items_in_correct_order);
 	suite_add_tcase(s, tc_dbsql);
 }
